""" This module is used to process text in docx, odt, txt and pdf files """

import re
import zipfile
from os import path

import pdfplumber
import slate3k as slate
from odf import text, teletype
from odf.opendocument import load


def get_file_extension(filepath: str) -> str:
    """Return the file extension of the file at the specified path"""
    if not path.isfile(filepath):
        print("Invalid file path")
        return ""

    try:
        return path.splitext(filepath)[1]
    except IndexError:
        print("File extension error")
        return ""


def file_extension_call(file: str) -> list:
    """Map file extension to appropriate function"""

    extension = get_file_extension(file)

    if extension:
        if extension == ".pdf":
            return get_words_from_pdf_file(file)
        if extension == ".docx":
            return get_words_from_docx_file(file)
        if extension == ".odt":
            return get_words_from_odt_file(file)
        if extension == ".txt":
            return get_words_from_txt_file(file)

    print("File format is not supported. Please convert to pdf, docx, odt or txt")
    return []


def get_words_from_pdf_file(pdf_path: str) -> list:
    """Return list of words from pdf file at specified path"""

    with open(pdf_path, "rb") as file:
        extracted_text = slate.PDF(file)

    nested_lists_length_sum = sum(len(temp) for temp in extracted_text)
    count_line_return = sum(string.count("\n") for string in extracted_text)

    # Check \n ratio compared to length of text
    if nested_lists_length_sum / count_line_return > 10:
        for i, _ in enumerate(extracted_text):
            extracted_text[i] = extracted_text[i].replace("\n", " ")
            extracted_text[i] = re.sub("<(.|\n)*?>", "", str(extracted_text[i]))
            extracted_text[i] = re.findall(r"\w+", extracted_text[i].lower())

        return [item for sublist in extracted_text for item in sublist]

    # Pdf format is not readable by Slate library
    return get_words_from_special_pdf(pdf_path)


def get_words_from_special_pdf(pdf_path: str) -> list:
    """Return list of words from a PDF file when the Slate library can't scrape it"""

    with pdfplumber.open(pdf_path) as file:
        concat_string = ""
        for page in file.pages:
            text_page = page.extract_text() + "\n"
            concat_string += text_page

    # Split the string into words and return as a list
    return concat_string.replace("\xa0", " ").strip().split()


def get_words_from_txt_file(txt_path: str) -> list:
    """Return list of words from txt file at specified path"""

    words = []

    with open(txt_path, encoding="utf-8") as file:
        for line in file:
            try:
                for word in line.split():
                    words.append(word.lower())
            except (UnicodeError, UnicodeDecodeError):
                pass

    str_words = " ".join(map(str, words))

    return re.findall(r"\w+", str_words)


def get_words_from_docx_file(docx_path: str) -> list:
    """Return list of words from docx file at specified path"""

    with zipfile.ZipFile(docx_path) as docx:
        content = docx.read("word/document.xml").decode("utf-8")
        cleaned = re.sub("<(.|\n)*?>", "", content)

    return re.findall(r"\w+", cleaned.lower())


def get_words_from_odt_file(odt_path: str) -> list:
    """Return list of words from odt file at specified path"""

    textdoc = load(odt_path)
    paragraphs = textdoc.getElementsByType(text.P)

    full_text = str()

    for paragraph in paragraphs:
        temp = teletype.extractText(paragraph)
        full_text += temp.lower()

    return re.findall(r"\w+", full_text)



""" This script stores functions for text processing, type validation, printing results

It verifies if value is float different from - 1
It prints similarity results in a pretty table in console
It waits for file creation
It can lemmatize, remove stop words, remove numbers for text processing

"""

import argparse
from os import path, listdir
from time import sleep
from typing import Any

from nltk.corpus import stopwords
from nltk.stem import WordNetLemmatizer


def parse_options():
    """
    Parses command-line arguments for the script.

    This function sets up an argument parser for the script, specifying the required input directory
    and optional output directory and block size arguments.

    Args:
    None

    Returns:
    argparse.Namespace: The parsed command-line arguments, where 'in_dir' is the input directory,
    'out_dir' is the optional output directory, and 'block_size' is the optional minimum number of
    consecutive similar words for block comparison (default is 2).
    """
    parser = argparse.ArgumentParser()
    parser.add_argument("in_dir", type=str, help="input directory for text files")
    parser.add_argument("-o", "--out_dir", type=str, help="output directory for html results files")
    parser.add_argument(
        "-s",
        "--block_size",
        type=int,
        help="minimum number of consecutive and " "similar words detected (default=2)",
    )

    return parser.parse_args()


def is_float(value: Any) -> bool:
    """Return true if value is a float and not equal to -1."""
    try:
        return float(value) != -1
    except ValueError:
        return False


def get_student_names(main_path):
    """
    Extracts student names from the directory names within the specified main path.

    This function assumes that each sub-directory within the main path represents a student and
    that the student's name is the first part of the sub-directory name, delimited by an underscore.

    Args:
    main_path (str): The path of the main directory containing student sub-directories.

    Returns:
    list: A list of student names extracted from the sub-directory names.
    """
    sub_directories = [name for name in listdir(main_path) if path.isdir(path.join(main_path, name))]

    return [title.split("_")[0] for title in sub_directories]


def pretty_table(scores: list, names: list) -> None:
    """Print similarity results nicely"""

    row_format = "{:>15}" * (len(names) + 1)
    print(row_format.format("", *names))
    for name, row in zip(names, scores):
        print(row_format.format(name, *row))


def wait_for_file(file_path: str, timeout: int = 10) -> bool:
    """Wait for the creation of a specific file.

    This method checks if the specified file exists and waits for it to
    appear during the specified amount of time (by default 10 seconds).

    filepath[in]    Fullpath to the file to wait for
    timeout[in]     Time to wait in seconds (by default 10 seconds).
    """

    attempts = 0
    while attempts < timeout:
        # Check if the file exists.
        if path.isfile(file_path):
            return True
        # Wait 1 second before trying again.
        sleep(1)
        attempts += 1

    return False


def remove_numbers(words_list: list) -> list:
    """Remove all numbers from strings list to avoid errors"""

    temp = [w for w in words_list if not isinstance(w, int)]
    return [w for w in temp if not isinstance(w, float)]


def remove_stop_words(words_list: list) -> list:
    """Remove stop words from strings list"""

    en_stop_words = set(stopwords.words("english"))

    return [w for w in words_list if str(w).lower() not in en_stop_words]


def lemmatize(words_list: list) -> list:
    """Return lemmatized words list"""

    lemmatizer = WordNetLemmatizer()

    return [lemmatizer.lemmatize(w) for w in words_list]



import subprocess
from setuptools import setup, find_packages


def get_version():
    return subprocess.check_output(["git", "describe", "--tags", "--abbrev=0"]).strip().decode("utf-8")[1:]


setup(
    name="copy-spotter",
    version=get_version(),
    packages=find_packages(),
    install_requires=[
        "beautifulsoup4==4.10.0",
        "nltk==3.6.6",
        "odfpy==1.4.1",
        "pdfplumber==0.5.28",
        "slate3k==0.5.3",
        "tabulate==0.8.9",
    ],
    extras_require={
        "lint": ["pylint==3.0.2", "mypy==1.7.1", "flake8==6.1.0", "black==24.3.0", "types-tabulate"],
        "dev": ["pytest", "pre-commit"],
    },
    author="Clément Delteil",
    author_email="clement45.delteil45@gmail.com",
    description="Make plagiarism detection easier. This package will find similar sentences between given files and "
    "highlight them in a side by side comparison.",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    url="https://github.com/Wazzabeee/copy-spotter",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python",
    ],
    python_requires=">=3.10",
    entry_points={
        "console_scripts": [
            "copy-spotter=scripts.main:main",
        ],
    },
)
